#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.")]
    public class FixedPercentTrailingStop : Indicator
    {
        #region Variables
		private int perc = 12;
		
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.Blue, PlotStyle.Line, "TrailingStop"));
            CalculateOnBarClose	= true;
            Overlay				= true;
			BarsRequired = 1;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 1)
				return;

			// Trailing stop
			double trail;
			double loss = Close[0] * ((double)Percent / 100);
			
			if (Close[0] > Value[1] && Close[1] > Value[1])
				trail = Math.Max(Value[1], Close[0] - loss);
			
			else if (Close[0] < Value[1] && Close[1] < Value[1])
				trail = Math.Min(Value[1], Close[0] + loss);
				
			else if (Close[0] > Value[1])
			{
				trail = Close[0] - loss;
				DrawArrowDown(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
			}
			
			else
			{
				trail = Close[0] + loss;
				DrawArrowUp(CurrentBar.ToString(), false, 1, Value[1], Color.Orange);
			}
			
            Value.Set(trail);
        }
		
		public override string ToString()
		{
			return Name + "(" + Percent + "%" + ")";
		}

		
        #region Properties
        [Description("Trailing Stop Percentage")]
        [Category("Parameters")]
        public int Percent
        {
            get { return perc; }
            set { perc = Math.Min(Math.Max(1, value), 30); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private FixedPercentTrailingStop[] cacheFixedPercentTrailingStop = null;

        private static FixedPercentTrailingStop checkFixedPercentTrailingStop = new FixedPercentTrailingStop();

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public FixedPercentTrailingStop FixedPercentTrailingStop(int percent)
        {
            return FixedPercentTrailingStop(Input, percent);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public FixedPercentTrailingStop FixedPercentTrailingStop(Data.IDataSeries input, int percent)
        {
            checkFixedPercentTrailingStop.Percent = percent;
            percent = checkFixedPercentTrailingStop.Percent;

            if (cacheFixedPercentTrailingStop != null)
                for (int idx = 0; idx < cacheFixedPercentTrailingStop.Length; idx++)
                    if (cacheFixedPercentTrailingStop[idx].Percent == percent && cacheFixedPercentTrailingStop[idx].EqualsInput(input))
                        return cacheFixedPercentTrailingStop[idx];

            FixedPercentTrailingStop indicator = new FixedPercentTrailingStop();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Percent = percent;
            indicator.SetUp();

            FixedPercentTrailingStop[] tmp = new FixedPercentTrailingStop[cacheFixedPercentTrailingStop == null ? 1 : cacheFixedPercentTrailingStop.Length + 1];
            if (cacheFixedPercentTrailingStop != null)
                cacheFixedPercentTrailingStop.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheFixedPercentTrailingStop = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FixedPercentTrailingStop FixedPercentTrailingStop(int percent)
        {
            return _indicator.FixedPercentTrailingStop(Input, percent);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.FixedPercentTrailingStop FixedPercentTrailingStop(Data.IDataSeries input, int percent)
        {
            return _indicator.FixedPercentTrailingStop(input, percent);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.FixedPercentTrailingStop FixedPercentTrailingStop(int percent)
        {
            return _indicator.FixedPercentTrailingStop(Input, percent);
        }

        /// <summary>
        /// Indicator as described in Sylvain Vervoort's 'Using Initial Stop Methods' May 2009 S&C article.
        /// </summary>
        /// <returns></returns>
        public Indicator.FixedPercentTrailingStop FixedPercentTrailingStop(Data.IDataSeries input, int percent)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.FixedPercentTrailingStop(input, percent);
        }

    }
}
#endregion
